@@ -15,6 +15,7 @@ import os
 import copy
 import math
 import itertools
+import functools
 import pkg_resources
 
 from io import open
@@ -25,6 +26,7 @@ from tokenize import untokenize, NUMBER, STRING, NAME, OP
 from .util import (formatter, logger, NUMERIC_TYPES, pi_theorem, solve_dependencies,
                    ParserHelper, string_types, ptok, string_preprocessor)
 
+from decimal import Decimal
 
 class UndefinedUnitError(ValueError):
     """Raised when the units are not defined in the unit registry.
@@ -147,20 +149,12 @@ class Definition(object):
 
     @property
     def symbol(self):
-        return self._symbol
+        return self._symbol or self._name
 
     @property
     def aliases(self):
         return self._aliases
 
-    @property
-    def name(self):
-        return self._name
-
-    @property
-    def symbol(self):
-        return self._symbol
-
     @property
     def converter(self):
         return self._converter
@@ -277,6 +271,12 @@ class UnitsContainer(dict):
         else:
             del self[key]
 
+    def __eq__(self, other):
+        if isinstance(other, string_types):
+            other = ParserHelper.from_string(other)
+            other = dict(other.items())
+        return dict.__eq__(self, other)
+
     def __str__(self):
         if not self:
             return 'dimensionless'
@@ -287,17 +287,13 @@ class UnitsContainer(dict):
         return '<UnitsContainer({})>'.format(tmp)
 
     def __format__(self, spec):
-        if spec == '!s' or spec == '':
-            return str(self)
-        elif spec == '!r':
-            return repr(self)
-        elif spec == '!l':
+        if 'L' in spec:
             tmp = formatter(self.items(), True, True,
                             r' \cdot ', r'\frac[{}][{}]', '{}^[{}]',
                             r'\left( {} \right)')
             tmp = tmp.replace('[', '{').replace(']', '}')
             return tmp
-        elif spec == '!p':
+        elif 'P' in spec:
             def fmt_exponent(num):
                 PRETTY = '⁰¹²³⁴⁵⁶⁷⁸⁹'
                 ret = '{:n}'.format(num).replace('-', '⁻')
@@ -308,8 +304,13 @@ class UnitsContainer(dict):
                             '·', '/', '{}{}',
                             '({})', fmt_exponent)
             return tmp
+        elif 'H' in spec:
+            tmp = formatter(self.items(), True, True,
+                            r' ', r'{}/{}', '{}<sup>{}</sup>',
+                            r'({})')
+            return tmp
         else:
-            raise ValueError('{} is not a valid format for UnitsContainer'.format(spec))
+            return str(self)
 
     def __copy__(self):
         ret = self.__class__()
@@ -412,9 +413,8 @@ class UnitRegistry(object):
         self.default_to_delta = default_to_delta
 
         if filename == '':
-            # Purposefully *not* using resource_stream as it may return a StringIO object for which we can't specify the encoding
-            data = pkg_resources.resource_string(__name__, 'default_en.txt').decode('utf-8')
-            self.load_definitions(data.splitlines())
+            data = pkg_resources.resource_filename(__name__, 'default_en.txt')
+            self.load_definitions(data)
         elif filename is not None:
             self.load_definitions(filename)
 
@@ -426,10 +426,17 @@ class UnitRegistry(object):
     def __getitem__(self, item):
         return self.parse_expression(item)
 
+    def __dir__(self):
+        return list(self._units.keys()) + \
+               ['define', 'load_definitions', 'get_name', 'get_symbol',
+                'get_dimensionality', 'Quantity', 'wraps', 'parse_unit',
+                'parse_units', 'parse_expression', 'pi_theorem',
+                'convert', 'get_base_units']
+
     def define(self, definition):
         """Add unit to the registry.
         """
-        if isinstance(definition, str):
+        if isinstance(definition, string_types):
             definition = Definition.from_string(definition)
 
         if isinstance(definition, DimensionDefinition):
@@ -488,6 +495,14 @@ class UnitRegistry(object):
             line = line.strip()
             if not line or line.startswith('#'):
                 continue
+            if line.startswith('@import'):
+                try:
+                    path = os.path.dirname(file.name)
+                except AttributeError:
+                    path = os.getcwd()
+                path = os.path.join(path, os.path.normpath(line[7:].strip()))
+                self.load_definitions(path)
+                continue
             try:
                 self.define(Definition.from_string(line))
             except Exception as ex:
@@ -635,13 +650,13 @@ class UnitRegistry(object):
         if src == dst:
             return value
         if len(src) == 1:
-            if not len(dst) == 1:
-                raise DimensionalityError(src, dst,
-                                          self.get_dimensionality(src),
-                                          self.get_dimensionality(dst))
             src_unit, src_value = list(src.items())[0]
             src_unit = self._units[src_unit]
             if not isinstance(src_unit.converter, ScaleConverter):
+                if not len(dst) == 1:
+                    raise DimensionalityError(src, dst,
+                                              self.get_dimensionality(src),
+                                              self.get_dimensionality(dst))
                 dst_unit, dst_value = list(dst.items())[0]
                 dst_unit = self._units[dst_unit]
                 if not type(src_unit.converter) is type(dst_unit.converter):
@@ -657,6 +672,11 @@ class UnitRegistry(object):
                                       self.get_dimensionality(src),
                                       self.get_dimensionality(dst))
 
+        # factor is type float and if our magintude is type Decimal then
+        # must first convert to Decimal before we can '*' the values
+        if isinstance(value, Decimal):
+            return Decimal(str(factor)) * value
+        
         return factor * value
 
     def pi_theorem(self, quantities):
@@ -781,6 +801,75 @@ class UnitRegistry(object):
                                          'Q_': self.Quantity,
                                          'U_': UnitsContainer})
 
+    def wraps(self, ret, args, strict=True):
+        """Wraps a function to become pint-aware.
+
+        Use it when a function requires a numerical value but in some specific
+        units. The wrapper function will take a pint quantity, convert to the units
+        specified in `args` and then call the wrapped function with the resulting
+        magnitude.
+
+        The value returned by the wrapped function will be converted to the units
+        specified in `ret`.
+
+        Use None to skip argument conversion.
+        Set strict to False, to accept also numerical values.
+
+        :param ret: output units.
+        :param args: iterable of input units.
+        :param strict: boolean to indicate that only quantities are accepted.
+        :return: the wrapped function.
+        :raises:
+            :class:`ValueError` if strict and one of the arguments is not a Quantity.
+        """
+
+        Q_ = self.Quantity
+
+        if not isinstance(args, (list, tuple)):
+            args = (args, )
+
+        def to_units(x):
+            if isinstance(x, string_types):
+                return self.parse_units(x)
+            elif isinstance(x, Q_):
+                return x.units
+            return x
+
+        units = [to_units(arg) for arg in args]
+
+        if isinstance(ret, (list, tuple)):
+            ret = ret.__class__([to_units(arg) for arg in ret])
+        elif isinstance(ret, string_types):
+            ret = self.parse_units(ret)
+
+        def decorator(func):
+            @functools.wraps(func)
+            def wrapper(*values, **kw):
+                new_args = []
+                for unit, value in zip(units, values):
+                    if unit is None:
+                        new_args.append(value)
+                    elif isinstance(value, Q_):
+                        new_args.append(self.convert(value.magnitude, value.units, unit))
+                    elif not strict:
+                        new_args.append(value)
+                    else:
+                        raise ValueError('A wrapped function using strict=True requires '
+                                         'quantity for all arguments with not None units. '
+                                         '(error found for {}, {})'.format(unit, value))
+
+                result = func(*new_args, **kw)
+
+                if isinstance(ret, (list, tuple)):
+                    return ret.__class__(res if unit is None else Q_(res, unit)
+                                         for unit, res in zip(ret, result))
+                elif ret is not None:
+                    return Q_(result, ret)
+
+                return result
+            return wrapper
+        return decorator
+
 
 def build_quantity_class(registry, force_ndarray=False):
     from .quantity import _Quantity
